Attribute VB_Name = "mod_ReadObjects"
    Option Explicit

Sub Initialize_ReadOutProcess(ByRef rMainFolder As MAPIFolder)

   'In dieser Prozedur werden alle EMails innerhalb des zuvor selektierten Verzeichnisses und all sei-
   'nen Unterverzeichnissen ausgelesen und in ein benutzderdefiniertes Array geschrieben. Dabei wird
   'unterschieden, ob das jeweilige EMail versendet oder empfangen wurde. Zudem werden smtliche Em-
   'pfnger, die fr die Archivierung bentigt werden, ausgelesen. Ebenfalls werden die bestehenden
   'Archiv-Verzeichnisse in ein benutzerdefiniertes Array geschrieben.

   'Definition der Konstanten fr entsprechende User Informationen.
    Const sMSG_FOLDER   As String = "Folgende Outlook Verzeichnisse konnten nicht ausgelesen werden:"
    Const sMSG_EMails   As String = "Es wurden keine E-Mail Objekte gefunden!" & vbCr & _
                                    "Prozess wird abgebrochen!"
    
    Dim strErrFolders   As String
        
   'Prozedur fr das Auslesen der Archiv-Verzeichnisse starten.
    Call Read_ArchiveFolders
    
   'Prozedur fr das Auslesen der EMails starten.
    Call Read_MailItems(rMainFolder, strErrFolders)
    
   'Prfen ob der Prozess vom User abgebrochen wurde.
    If gl_Cancel = Abort.Interrupted Then
       'Falls Abbruch erwnscht, UserForm entladen und Prozedur beenden.
        Unload frm_ReadObjects
        Exit Sub
    Else
       'Da der Lese-Prozess jetzt beendet ist, "Abbruch" Flag auf True setzen.
        gl_Cancel = Abort.UserEditing
    End If
    
   'Prfen, ob beim auslesen der EMail-Objekte ein Fehler aufgetreten ist.
    If strErrFolders <> "" Then
       'Alle Verzeichnisse die nicht gelesen werden konnten anzeigen.
        MsgBox sMSG_FOLDER & strErrFolders, vbExclamation, "Lesefehler"
    End If
    
   'Prfen ob EMail-Objekte ausgelesen wurden.
    If UBound(ga_Mails) = 0 Then
       
       'Info anzeigen, dass keine EMail-Objekte geladen wurden.
        MsgBox sMSG_EMails, vbExclamation, "Abbruch"
       
       'Menbutton wieder aktivieren.
        g_CmdButton.Enabled = True
        
       'UserForm entladen und somit Prozess beenden.
        Unload frm_ReadObjects
        
    Else
    
       'Falls EMail-Objekte ausgelesen wurden, Steuerelemente auf der UserForm aktivieren.
        With frm_ReadObjects
            .Caption = " Alle EMail-Objekte wurden ausgelesen!"
            .cmdNext.Enabled = True
            .cmdShow.Enabled = True
            .frameItems.Enabled = True
            .cmdNext.SetFocus
        End With
        
    End If

End Sub

Private Sub Read_ArchiveFolders()

   'In dieser Prozedur werden smtliche bereits bestehenden Archive-Verzeichnisse ausgelesen und danach
   'in ein benutzerdefiniertes Array geschrieben. Die Archiv-Verzeichnisse werden aufgrund der Eigen-
   'schaft 'Description' identifiziert.
   
    Dim strError    As String
    Dim objArchiv   As MAPIFolder
    
   'Fehlerbehandlungs-Routine initialisieren.
    On Error GoTo Err_ReadFolder
    
   'Schleife fr alle direkten Unterverzeichnisse in MAPI Namespace initialisieren.
    For Each objArchiv In Application.Session.Folders
       
       'Prfen ob es sich um ein Archiv-Verzeichnis handelt.
        If InStr(1, objArchiv.Description, gs_DESCRIPTION, vbBinaryCompare) <> 0 Then
        
           'Archiv-Verzeichnis in benutzerdefiniertes Array schreiben.
            Call Add_ArchiveFolder(objArchiv)
            
        End If
                
Res_NextFolder:
        
    Next objArchiv

   'Prfen ob ein Laufzeitfehler aufgetreten ist.
    If strError <> "" Then
    
       'Laufzeitfehler in Messagebox anzeigen.
        MsgBox "Fehler beim Auslesen von Outlook Verzeichnissen:" & strError, vbCritical, ""
        
    End If

   'Prozedur beenden.
    Exit Sub
    
Err_ReadFolder:

   'Name des Archiv-Verzeichnisses und Angaben des Laufzeitfehlers in Variable schreiben.
    strError = strError & String(2, vbCr) & _
               "Name:  " & vbTab & objArchiv.Name & vbCr & _
               "Quelle:" & vbTab & Err.Number & " - " & Err.Source & vbCr & _
               "Fehler:" & vbTab & Err.Description

   'Code-Ausfhrung mit nchstem Outlook Verzeichnis fortsetzen.
    Resume Res_NextFolder
    
End Sub

Public Sub Add_ArchiveFolder(ByRef rArchive As MAPIFolder)

   'In dieser Prozedur wird geprft, ob das Archiv-Verzeichnis in der Description Eigenschaft ein Ar-
   'chivierungs-Zeitraum beinhaltet. Falls dem so ist, werden die notwendigen Eigenschaften des Verzei-
   'chnisses in ein benutzerdefiniertes Array geschrieben.

    Dim strTimeSpan() As String
    
   'Die 'Description' Eigenschaft auf die einzelnen Zeilen aufteilen.
    strTimeSpan = Split(rArchive.Description, vbCrLf)
    
   'Prfen ob Archivierungszeitraum und Typenbezeichnung vorhanden sind.
    If UBound(strTimeSpan) <> 2 Then Exit Sub

   'Prfen ob es sich um eine Datums-Angabe handelt.
    If IsDate(strTimeSpan(1)) And IsDate(strTimeSpan(2)) Then

       'Benutzerdefiniertes Array um eine Position erweitern.
        ReDim Preserve ga_Archives(UBound(ga_Archives) + 1) As Archive_TYPE
    
       'Daten des Archiv-Verzeichnisses in benutzerdefiniertes Array schreiben.
        With ga_Archives(UBound(ga_Archives))
            .sFolderName = rArchive.Name    'Verzeichnisname
            .sEntryID = rArchive.EntryID    'EntryID
            .sStoreID = rArchive.StoreID    'StoreID
            .dtFrom = CDate(strTimeSpan(1)) 'Datum von
            .dtTo = CDate(strTimeSpan(2))   'Datum bis
        End With
        
    End If

End Sub

Private Sub Read_MailItems(ByRef rFolder As MAPIFolder, ByRef rsError As String)
    
   'In dieser Prozedur werden smtliche Entry- und StoreIDs der EMails aus allen Unterverzeichnissen
   'des zuvor selektierten Verzeichnisses (aus dem Pickfolder Dialog) in ein 1 dimenisonales benutzer-
   'definiertes Array eingelesen. Damit kann spter das EMail-Objekt auf einfache Weise wieder identi-
   'fizert werden.
    
    Dim objItem      As Object
    Dim objSubFolder As MAPIFolder
    
   'Fehler-Routine fr Verzeichnisse die ausgelesen werden initialisieren.
    On Error GoTo Err_ReadFolder
    
   'Schleife fr alle EMails des aktuellen Verzeichnisses initialisieren.
    For Each objItem In rFolder.Items
        
       'Prfen ob der Anwender den Abbruch des Prozesses besttigt hat.
        If gl_Cancel = Abort.Interrupted Then Exit Sub
        
       'Fehler-Routine fr EMails die ausgelesen werden initialisieren.
        On Error GoTo Err_ReadItem
        
       'Klasse des Objektes berprfen.
        Select Case objItem.Class
            
           'Bezeichnung der Konstanten
           '43 = olMail
           '53 = olMeetingRequest
           '54 = olMeetingCancellation
           '55 = olMeetingResponseNegative
           '56 = olMeetingResponsePositive
           '57 = olMeetingResponseTentative
           
           'Objekt archivieren, falls es sich um ein Mail- oder Meeting-Objekt handelt.
            Case 43, 53 To 57
        
               'Prfen, ob das aktuelle EMail bereits gelesen wurde.
                If Not objItem.UnRead Then
                    
                   'Prozedur fr das Auslesen der EMail Eigenschaften aufrufen.
                    Call Add_MailItem(rFolder, objItem)
                   
                   'Prozedur fr die Aktualisierung der Listbox aufrufen.
                    Call Show_MailItem(rFolder.Name, objItem.Subject)
                    
                End If
        End Select
        
Res_NextItem:
        
    Next objItem
             
   'Fehler-Routine fr Verzeichnisse die ausgelesen werden initialisieren.
    On Error GoTo Err_ReadFolder
    
   'Schleife fr alle direkten Unterverzeichnisse initialisieren.
    For Each objSubFolder In rFolder.Folders
        
       'Prfen ob der Anwender den Abbruch des Prozesses besttigt hat.
        If gl_Cancel = Abort.Interrupted Then Exit Sub
        
       'Prozedur nochmals aufrufen, um die restlichen Unterverzeichnisse auszulesen.
        Call Read_MailItems(objSubFolder, rsError)
        
    Next objSubFolder

   'Prozedur beenden.
    Exit Sub

Err_ReadItem:

   'Falls das Auslesen eines EMails zu einem Laufzeitfehler fhrt, wird geprft, ob fr dieses bereits
   'eine Position im Array erstellt wurde. Falls ja, wird diese Position wieder gelscht. Danach wird
   'die Schleife mit dem nchsten EMail fortgesetzt.
     
   'Prfen ob Array schon mind. 1mal erweitert wurde.
    If UBound(ga_Mails) > 0 Then
       'Prfen ob Array fr das fehlerauslsende EMail schon erweitert wurde.
        If ga_Mails(UBound(ga_Mails)).sEntryID = "" Then
           'Position fr das fehlerauslsende EMail wieder lschen.
            ReDim Preserve ga_Mails(UBound(ga_Mails) - 1) As Mail_TYPE
        End If
    End If
        
   'Code-Ausfhrung mit nchstem EMail fortsetzen.
    Resume Res_NextItem
        
Err_ReadFolder:

   'Falls ein Verzeichnis nicht ausgelesen werden kann, wird in dieser Variable der Verzeichnis-Name
   'gespeichert. Kann beispielsweise vorkommen, wenn man keine Zugriffsberechtigung oder keine Verbin-
   'dung (Offline-Modus) zum Exchange Server hat.
   
    rsError = rsError & String(2, vbCr) & _
              "Name:  " & vbTab & rFolder.Name & vbCr & _
              "Quelle:" & vbTab & Err.Number & " - " & Err.Source & vbCr & _
              "Fehler:" & vbTab & Err.Description
    
End Sub

Private Sub Add_MailItem(ByRef rFolder As MAPIFolder, ByRef rItem As Object)
    
   'In dieser Prozedur werden die notwendigen Eigenschaften des EMails in ein benutzerdefiniertes Array
   'geschrieben, welche spter fr die Archivierung benutzt werden. Dabei wird unterschieden, ob es
   'sich um ein gesendetes oder empfangenes EMail handelt.
    
    Dim strAdr      As String
    Dim objRecip    As Recipient
    
   'Benutzerdefiniertes Array um eine Position erweitern.
    ReDim Preserve ga_Mails(UBound(ga_Mails) + 1) As Mail_TYPE

    With ga_Mails(UBound(ga_Mails))
        
       'Arrays fr Speicherung der Namen und Adressen initialisieren.
        ReDim .saName(0) As String
        ReDim .saAddress(0) As String
        
       'EntryID und StoreID des EMails in Array schreiben.
        .sEntryID = rItem.EntryID
        .sStoreID = rFolder.StoreID
        
       'Prfen ob das EMail gesendet oder empfangen wurde.
        If UCase(rItem.SenderName) <> UCase(Application.Session.CurrentUser) Then
            
           'Das EMail wurde empfangen! In diesem Fall wird das EMail nur in das Verzeichnis des Senders
           'archiviert, unabhngig davon, ob mehrere Empfnger das EMail erhalten haben.
            
           'Name und Adresse des Senders des EMails ohne Apostroph in Array schreiben.
            .saName(0) = Replace(rItem.SenderName, "'", "")
            .saAddress(0) = Replace(UCase(rItem.SenderEmailAddress), "'", "")
            
        Else
            
           'Das EMail wurde gesendet! In diesem Fall werden smtliche Empfnger des EMails ausgelesen.
           'Es werden alle Felder ausgelesen (To, Cc und Bcc).
    
           'EMail kennzeichnen, dass es versendet und nicht empfangen wurde.
            .bSentMySelf = True
    
           'Schleife fr alle Empfnger initialisieren.
            For Each objRecip In rItem.Recipients
            
               'berprfen ob der Empfnger doppelt vorhanden ist.
                If InStr(1, strAdr, vbTab & objRecip.Address, vbBinaryCompare) = 0 Then
                    
                   'Name und Adresse des Empfngers in benutzerdefiniertes Array schreiben.
                    .saName(UBound(.saName)) = Replace(objRecip.Name, "'", "")
                    .saAddress(UBound(.saAddress)) = Replace(UCase(objRecip.Address), "'", "")
                    
                   'String erweitern, fr die berprfung, ob Empfnger mehrfach vorhanden ist.
                    strAdr = strAdr & vbTab & .saAddress(UBound(.saAddress))
                    
                   'Arrays um eine Position erweitern.
                    ReDim Preserve .saName(UBound(.saName) + 1) As String
                    ReDim Preserve .saAddress(UBound(.saAddress) + 1) As String
                    
                End If
                
            Next objRecip
            
           'berflssige letzte Position im Array entfernen.
            ReDim Preserve .saName(UBound(.saName) - 1) As String
            ReDim Preserve .saAddress(UBound(.saAddress) - 1) As String
            
        End If
        
    End With

End Sub

Private Sub Show_MailItem(ByVal vsFolderName As String, ByVal vsSubject As String)
    
   'In dieser Prozedur wird laufend das aktuell ausgelesene EMail-Objekt in der Listbox angezeigt. Mit
   'DoEvents wird erreicht, dass der User whrend des Prozesses den Fortschritt mitverfolgen kann.

   With frm_ReadObjects
        
       'Einzelne Informationen des EMails in die Listbox einfgen.
        With .listItems
            .AddItem                                'Zeile in Listbox hinzufgen
            .List(.ListCount - 1, 0) = vsFolderName 'Verzeichnisname hinzufgen
            .List(.ListCount - 1, 1) = vsSubject    'EMail-Betreff hinzufgen
            .TopIndex = .ListCount                  'Listbox autom. mitscrollen
        End With
        
        'Angabe der Anzahl gelesenen EMail-Objekte aktualisieren.
        .txtCount.Text = Format(UBound(ga_Mails), "#,##0")
        
    End With
    
   'Prozess freigeben.
    DoEvents

End Sub
